@@ -25,7 +25,7 @@
  * @author     Yomei Komiya
  * @copyright  2007 the original author or authors.
  * @license    http://www.apache.org/licenses/LICENSE-2.0  Apache License 2.0
- * @version    SVN: $Id: ToStringStyle.php 253 2008-03-08 04:19:45Z whitestar $
+ * @version    SVN: $Id: ToStringStyle.php 351 2008-09-02 12:33:19Z whitestar $
  * @link       http://phpcommons.sourceforge.jp/
  * @see        
  * @since      File available since Release 0.8.0
@@ -35,6 +35,7 @@
 
 require_once 'Commons/Lang/Object.php';
 require_once 'Commons/Lang/ClassUtils.php';
+require_once 'Commons/Lang/ObjectUtils.php';
 require_once 'Commons/Lang/StringBuffer.php';
 require_once 'Commons/Lang/StringUtils.php';
 require_once 'Commons/Lang/System.php';
@@ -43,6 +44,7 @@ require_once 'Commons/IO/Serializable.php';
 /*
 use Commons::Lang::Object;
 use Commons::Lang::ClassUtils;
+use Commons::Lang::ObjectUtils;
 use Commons::Lang::StringBuffer;
 use Commons::Lang::StringUtils;
 use Commons::Lang::System;
@@ -59,7 +61,7 @@ use Commons::IO::Serializable;
  * @author     Yomei Komiya
  * @copyright  2007 the original author or authors.
  * @license    http://www.apache.org/licenses/LICENSE-2.0  Apache License 2.0
- * @version    Release: 1.0.2
+ * @version    Release: 1.0.3
  * @link       http://phpcommons.sourceforge.jp/
  * @see        
  * @since      Class available since Release 0.8.0
@@ -93,6 +95,9 @@ abstract class Commons_Lang_Builder_ToStringStyle
 	}
 	
 	
+	private $_registry = array();
+	private $_nestLevel = 0;
+	private $_maxNestLevel = 10;
 	private $_useFieldNames = true;
 	private $_useClassName = true;
 	private $_useShortClassName = false;
@@ -127,6 +132,33 @@ abstract class Commons_Lang_Builder_ToStringStyle
 		// do nothing.
 	}
 	
+/*	
+	protected function isRegistered($value) {
+		foreach ($this->_registry as $object) {
+			if ($object === $value) {
+				return true;
+			}
+		}
+		return false;
+	}
+	
+	
+	protected function register($value) {
+		if (!is_null($value)) {
+			$this->_registry[] = $value;
+		}
+	}
+	
+	
+	protected function unregister($value) {
+		foreach ($this->_registry as $key => $object) {
+			if ($object === $value) {
+				unset($this->_registry[$key]);
+				return;
+			}
+		}
+	}
+*/	
 	
 	/**
 	 * Appends super's toString
@@ -261,7 +293,13 @@ abstract class Commons_Lang_Builder_ToStringStyle
 	 */
 	protected function appendInternal(
 		Commons_Lang_StringBuffer $buffer, $fieldName, $value, $detail) {
-
+		if ($this->_nestLevel > $this->_maxNestLevel) {
+			$this->appendDeepNestCollection($buffer, $fieldName, $value);
+			return;
+		}
+		
+		$this->_nestLevel++;
+			
 		if (is_array($value)) {
 			if ($detail) {
 				$this->appendDetail($buffer, $fieldName, $value);
@@ -295,6 +333,41 @@ abstract class Commons_Lang_Builder_ToStringStyle
 				$this->appendSummary($buffer, $fieldName, $value);
 			}
 		}
+		
+		$this->_nestLevel--;
+	}
+	
+	
+	/**
+	 * Appends deep nest collection(array).
+	 *
+	 * @param Commons_Lang_StringBuffer $buffer
+	 * @param string $fieldName
+	 * @param mixed $value
+	 */
+	protected function appendDeepNestCollection($buffer, $fieldName, $value) {
+		if (is_null($value)) {
+			$value = $this->_nullText;
+		}
+		
+		if (is_array($value)) {
+			$buffer->append('*DEEP NEST*...');
+		}
+		else {
+			$buffer->append((string)$value);
+		}
+	}
+	
+
+	/**
+	 * Appends cyclic object
+	 *
+	 * @param Commons_Lang_StringBuffer $buffer
+	 * @param string $fieldName
+	 * @param mixed $value
+	 */
+	protected function appendCyclicObject($buffer, $fieldName, $value) {
+		Commons_Lang_ObjectUtils::appendIdentityToString($buffer, $value);
 	}
 	
 	
@@ -344,7 +417,7 @@ abstract class Commons_Lang_Builder_ToStringStyle
 					$buffer->append($this->_arraySeparator);
 				}
 				$buffer->append('[' . $key . ']=>');
-				$this->appendDetail($buffer, $fieldName, $elm);
+				$this->appendInternal($buffer, $fieldName, $elm, $this->_arrayContentDetail);
 				
 				$i++;
 			}
@@ -445,6 +518,22 @@ abstract class Commons_Lang_Builder_ToStringStyle
 	
 	
 	// Accessors ---------------------------------------------------------------------------
+	/**
+	 * @return int Expanded nest level of array. The default level is 10.
+	 * @since 1.0.3
+	 */
+	public function getMaxNestLevel() {
+		return $this->_maxNestLevel;
+	}
+	
+	/**
+	 * @param int $nestLevel
+	 * @since 1.0.3
+	 */
+	public function setMaxNestLevel($nestLevel) {
+		$this->_maxNestLevel = $nestLevel;
+	}
+	
 	protected function isUseClassName() {
 		return $this->_useClassName;
 	}
@@ -654,7 +743,7 @@ abstract class Commons_Lang_Builder_ToStringStyle
  * @author     Yomei Komiya
  * @copyright  2007 the original author or authors.
  * @license    http://www.apache.org/licenses/LICENSE-2.0  Apache License 2.0
- * @version    Release: 1.0.2
+ * @version    Release: 1.0.3
  * @link       http://phpcommons.sourceforge.jp/
  * @see        
  * @since      Class available since Release 0.8.0
@@ -684,7 +773,7 @@ final class Commons_Lang_Builder_DefaultToStringStyle
  * @author     Yomei Komiya
  * @copyright  2007 the original author or authors.
  * @license    http://www.apache.org/licenses/LICENSE-2.0  Apache License 2.0
- * @version    Release: 1.0.2
+ * @version    Release: 1.0.3
  * @link       http://phpcommons.sourceforge.jp/
  * @see        
  * @since      Class available since Release 0.8.0
@@ -714,7 +803,7 @@ final class Commons_Lang_Builder_NoFieldNameToStringStyle
  * @author     Yomei Komiya
  * @copyright  2007 the original author or authors.
  * @license    http://www.apache.org/licenses/LICENSE-2.0  Apache License 2.0
- * @version    Release: 1.0.2
+ * @version    Release: 1.0.3
  * @link       http://phpcommons.sourceforge.jp/
  * @see        
  * @since      Class available since Release 0.8.0
@@ -746,7 +835,7 @@ final class Commons_Lang_Builder_ShortPrefixToStringStyle
  * @author     Yomei Komiya
  * @copyright  2007 the original author or authors.
  * @license    http://www.apache.org/licenses/LICENSE-2.0  Apache License 2.0
- * @version    Release: 1.0.2
+ * @version    Release: 1.0.3
  * @link       http://phpcommons.sourceforge.jp/
  * @see        
  * @since      Class available since Release 0.8.0
@@ -781,7 +870,7 @@ final class Commons_Lang_Builder_SimpleToStringStyle
  * @author     Yomei Komiya
  * @copyright  2007 the original author or authors.
  * @license    http://www.apache.org/licenses/LICENSE-2.0  Apache License 2.0
- * @version    Release: 1.0.2
+ * @version    Release: 1.0.3
  * @link       http://phpcommons.sourceforge.jp/
  * @see        
  * @since      Class available since Release 0.8.0
